﻿using System;
using System.Collections.Generic;
using System.Text;

namespace NGinnBPM.Runtime.Events
{
    /// <summary>
    /// Event used in system diagnostics.
    /// Application components send it to notify about their status.
    /// </summary>
    [Serializable]
    public class DiagnosticEvent
    {
        public enum EventSeverity
        {
            Info,
            Warn,
            Error
        }

        public string Message { get; set; }
        public string Category { get; set; }
        public string ErrorInfo { get; set; }
        public DateTime Timestamp { get; set; }
        public EventSeverity Severity { get; set; }
        public string TaskInstanceId { get; set; }

        public DiagnosticEvent()
        {
            Timestamp = DateTime.Now;
        }

        public DiagnosticEvent(EventSeverity severity, string category, string message)
        {
            Message = message;
            Category = category;
            Severity = severity;
            Timestamp = DateTime.Now;
        }

        public DiagnosticEvent(string category, Exception error)
        {
            Timestamp = DateTime.Now;
            Severity = EventSeverity.Error;
            Message = error.Message;
            ErrorInfo = error.ToString();
            Category = category;
        }
    }
}
