﻿using System;
using System.Collections.Generic;
using System.Text;

namespace NGinnBPM.Services
{
    /// <summary>
    /// Registry that maps some ID to task correlation ID.
    /// Used for routing messages to proper ReceiveMessageTasks.
    /// For example: we send out email with process instanceID in subject and 
    /// set up ReceiveMessageTask that will wait for the response.
    /// Email response contains original email's subject with embedded process instanceId,
    /// so we can find proper receiveMessageTask with this instanceId (provided
    /// that there's only one receiveMessageTask for that process)
    /// Mappings held by resolver are persistent.
    /// </summary>
    public interface IMessageCorrelationIdResolver
    {
        /// <summary>
        /// register mapping of some ID to task instance id
        /// Id must be unique.
        void RegisterMapping(string messageCorrelationId, string receiveTaskInstanceId);
        string GetReceiveTaskInstanceId(string messageCorrelationId);
        void RemoveMapping(string messageCorrelationId, string receiveTaskInstanceId);
        void RemoveAllProcessMappings(string processInstanceId);
    }
}
