﻿using System;
using System.Collections.Generic;
using System.Text;
using NGinnBPM.Lib.Schema;
using NGinnBPM.Lib.Data;
using NGinnBPM.Runtime;
using NGinnBPM.Lib.Interfaces;


namespace NGinnBPM.Services
{
    
    /// <summary>
    /// Interface for evaluating task scripts
    /// - data and parameter bindings
    /// - script task's body
    /// This interface is introduced to avoid coupling to single script evaluation engine
    /// and to isolate all script invocations.
    /// </summary>
    public interface ITaskScript
    {
        /// <summary>
        /// Script source data
        /// </summary>
        Dictionary<string, object> SourceData
        {
            get;
            set;
        }

       
        

        /// <summary>
        /// Task context
        /// </summary>
        ITaskContext TaskContext
        {
            get;
            set;
        }

        /// <summary>
        /// Task instance
        /// </summary>
        TaskInstance TaskInstance
        {
            get;
            set;
        }




        /// <summary>
        /// Get value of task input parameter
        /// </summary>
        /// <param name="paramName">Parameter name</param>
        /// <returns></returns>
        object GetInputParameterValue(string paramName);

        /// <summary>
        /// Calculate default value of task variable
        /// </summary>
        /// <param name="variableName"></param>
        /// <returns></returns>
        object GetDefaultVariableValue(string variableName);


        /// <summary>
        /// Evaluate script block
        /// </summary>
        /// <param name="blockId">Id of the block (not the script code)</param>
        /// <returns></returns>
        object RunScriptBlock(string blockId);

        /// <summary>
        /// Evaluate task input variable binding.
        /// Source data should be set to process variables.
        /// </summary>
        /// <param name="varName"></param>
        /// <returns></returns>
        object EvalInputVariableBinding(string varName);

        /// <summary>
        /// Evaluate task output data binding.
        /// </summary>
        /// <param name="varName"></param>
        /// <returns></returns>
        object EvalOutputVariableBinding(string varName);

        /// <summary>
        /// Evaluate binding of task output parameters to task output variable.
        /// </summary>
        /// <param name="varName"></param>
        /// <returns></returns>
        object EvalOutputParameterBinding(string varName);

        /// <summary>
        /// Evaluate multi instance task split query
        /// </summary>
        /// <returns></returns>
        object EvalMultiInstanceSplitQuery();

        /// <summary>
        /// Evaluate input condition of given flow.
        /// Flow must join two nodes in current composite task.
        /// </summary>
        /// <param name="fl"></param>
        /// <returns></returns>
        bool EvaluateFlowInputCondition(Flow fl);
    }
}
