﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.Collections;

namespace BasicTests.Ser
{
    
    [DataContract]
    [KnownType(typeof(Arr))]
    [KnownType(typeof(RecV))]
    [KnownType(typeof(StrV))]
    [KnownType(typeof(IntV))]
    [KnownType(typeof(DblV))]
    [KnownType(typeof(BoolV))]
    [KnownType(typeof(DtV))]
    public abstract class V
    {
        public abstract object GetValue();

        public static V Wrap(object d)
        {
            if (d == null) return null;
            if (d is V) return d as V;
            if (d is string) return new StrV { Val = d as string };
            if (d is int) return new IntV { Val = (int)d };
            if (d is double || d is float) return new DblV { Val = (double)d };
            if (d is bool) return new BoolV { Val = (bool)d };
            if (d is DateTime) return new DtV { Val = (DateTime)d };

            if (typeof(IDictionary<string, object>).IsAssignableFrom(d.GetType()))
            {
                IDictionary<string, object> dic = d as IDictionary<string, object>;
                var ret = new RecV();
                ret.Items = new List<Fld>();
                foreach (string k in dic.Keys)
                    ret.Items.Add(new Fld { Name = k, Val = V.Wrap(dic[k]) });
                return ret;
            }
            if (d is IEnumerable)
            {
                var en = d as IEnumerable;
                var ret = new Arr();
                ret.Items = new List<V>();
                foreach (var v in en)
                    ret.Items.Add(V.Wrap(v));
                return ret;
            }
            

            throw new NotImplementedException("V: " + d);
        }
    }

    [DataContract]
    public class Arr : V
    {
        [DataMember]
        public List<V> Items { get; set; }
        public override object GetValue()
        {
            return Items;
        }
    }

    [DataContract]
    public class Fld
    {
        [DataMember]
        public string Name { get; set; }
        [DataMember]
        public V Val { get; set; }
    }

    [DataContract]
    public class RecV : V
    {
        [DataMember]
        public List<Fld> Items { get; set; }
        public override object GetValue()
        {
            return Items;
        }

        
    }

    [DataContract]
    public class StrV : V
    {
        [DataMember]
        public string Val { get; set; }
        public override object GetValue()
        {
            return Val;
        }
    }

    [DataContract]
    public class IntV : V
    {
        [DataMember]
        public int Val { get; set; }
        public override object GetValue()
        {
            return Val;
        }
    }

    [DataContract]
    public class DblV : V
    {
        [DataMember]
        public double Val { get; set; }
        public override object GetValue()
        {
            return Val;
        }
    }

    [DataContract]
    public class DtV : V
    {
        [DataMember]
        public DateTime Val { get; set; }
        public override object GetValue()
        {
            return Val;
        }
    }

    [DataContract]
    public class BoolV : V
    {
        [DataMember]
        public bool Val { get; set; }
        public override object GetValue()
        {
            return Val;
        }
    }


}
