﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NGinnBPM.Lib.Data;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using NLog;
using System.IO;

namespace NGTestApp2
{
    [DataContract]
    public abstract class V
    {
        [IgnoreDataMember]
        public abstract object Value { get; }
    }

    [DataContract]
    public class IntV : V
    {
        [DataMember]
        public int V { get; set; }

        public override object Value
        {
            get { return V; }
        }
    }

    [DataContract]
    public class DoubleV : V
    {
        [DataMember]
        public double V { get; set; }
        public override object Value
        {
            get { return V; }
        }
    }

    [DataContract]
    public class StringV : V
    {
        [DataMember]
        public string V { get; set; }
        public override object Value
        {
            get { return V; }
        }
    }

    [DataContract]
    public class ArrV : V
    {
        [DataMember]
        public V[] V { get; set; }
        public override object Value
        {
            get { return V; }
        }
    }

    [DataContract]
    public class MapV : V
    {
        [DataMember]
        public Dictionary<string, V> V { get; set; }
        public override object Value
        {
            get { return V; }
        }
    }



    [DataContract]
    public class DataContainer : IDynData
    {
        [IgnoreDataMember]
        public ICollection<string> FieldNames
        {
            get { return _data.Keys; }
        }

        public object this[string idx]
        {
            get
            {
                return _data[idx];
            }
            set
            {
                _data[idx] = value;
            }
        }

        private Dictionary<string, object> _data = new Dictionary<string, object>();

        [DataMember]
        public Dictionary<string, object> Data
        {
            get { return _data; }
            set { _data = value; }
        }

        public class KV
        {
            public string Name { get; set; }
            public object Value { get; set; }
        }

        [DataMember]
        public KV[] D
        {
            get
            {
                List<KV> lst = new List<KV>();
                foreach (string k in _data.Keys)
                    lst.Add(new KV { Name = k, Value = _data[k] });
                return lst.ToArray();
            }
            set
            {
                _data = new Dictionary<string, object>();
                foreach (KV v in value)
                    _data[v.Name] = v.Value;
            }
        }

        
        
    }

    public class DataContainerSerialization
    {
        public static void Test1()
        {

            DataContainer dc = new DataContainer();
            dc["ala"] = "ma kota";
            DataContainer dc2 = new DataContainer();
            dc2["kot"] = "ma alę";
            dc2["wiek"] = 15;
            dc2["cosie"] = new int[] {1, 2, 3, 4, 6};
            dc["juzio"] = dc2;
            dc["dzis"] = DateTime.Now;

            StringWriter sw = new StringWriter();
            JsonSerializer ser = new JsonSerializer();
            ser.Serialize(new JsonTextWriter(sw), dc);
            Console.WriteLine("Serialized 1: {0}", sw.ToString());

            DataContainer dc3 = ser.Deserialize<DataContainer>(new JsonTextReader(new StringReader(sw.ToString())));

            NetDataContractSerializer ser2 = new NetDataContractSerializer();
            MemoryStream ms = new MemoryStream();
            ser2.Serialize(ms, dc);
            ms.Seek(0, SeekOrigin.Begin);
            StreamReader sr = new StreamReader(ms, true);
            Console.WriteLine("XML: {0}", sr.ReadToEnd());

            ms = new MemoryStream();
            System.Runtime.Serialization.Json.DataContractJsonSerializer ser3 = new System.Runtime.Serialization.Json.DataContractJsonSerializer(typeof(DataContainer));
            ser3.WriteObject(ms, dc);
            ms.Seek(0, SeekOrigin.Begin);
            sr = new StreamReader(ms, true);
            Console.WriteLine("XML: {0}", sr.ReadToEnd());



        }
    }
}
