﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NLog;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using NGinnBPM.Rules;
using System.IO;
using System.Runtime.Serialization;
using System.Collections;
using NGinnBPM.Lib.Data;

namespace NGTestApp2
{
    public class JObjectWrapper : IDynObject
    {
        
        private JObject _job;

        public JObjectWrapper(JObject job)
        {
            _job = job;
        }

        public JObjectWrapper()
        {
            _job = new JObject();
        }

        public object Get(string key)
        {
            JToken tok;
            if (!_job.TryGetValue(key, out tok))
                throw new Exception("Member not defined: " + key);
            switch(tok.Type)
            {
                case JTokenType.Date:
                case JTokenType.Boolean:
                case JTokenType.Float:
                case JTokenType.Integer:
                case JTokenType.String:
                    return tok;
                case JTokenType.Null:
                case JTokenType.Undefined:
                    return null;
                case JTokenType.Array:
                    throw new NotImplementedException();
                case JTokenType.Object:
                    throw new NotImplementedException();
                default:
                    throw new NotImplementedException();
            }
        }

        public object Invoke(string member, object[] arguments)
        {
            throw new NotImplementedException();
        }

        public void Set(string key, object value)
        {
            _job[key] = JToken.FromObject(value);
        }

        protected static object ToAcceptableValue(object v)
        {
            return v;
        }

        public ICollection<string> GetFieldNames()
        {
            throw new NotImplementedException();
        }
    }

    [DataContract]
    public class DicWrap : IDynObject
    {
        public DicWrap()
        {
            Data = new Dictionary<string, object>();
        }

        public object Get(string key)
        {
            object v;
            return Data.TryGetValue(key, out v) ? v : null;
        }

        public ICollection<string> GetFieldNames()
        {
            return Data.Keys;
        }

        public object Invoke(string member, object[] arguments)
        {
            throw new NotImplementedException();
        }

        public void Set(string key, object value)
        {
            Data[key] = value;
        }

        [DataMember]
        public Dictionary<string, object> Data { get; set; }
    }

    public class SerializationTest
    {
        private static JsonSerializer ser = new JsonSerializer();

        public static void Test()
        {
            string json = @"{'ala': 'ma kota', 'kot': 188, 'ziutek': {imie: 'ziut', nazwisko: 'dwaziuty'}, aLista: [1, 2, 3, {name: 'a ty tu skad?'}]}";

            JObject o1 = (JObject) ser.Deserialize(new JsonTextReader(new StringReader(json)));

            Console.WriteLine("Now dynobj!");
            DynObject dob = DynObject.FromJObject(o1);

            Dictionary<string, object> o2 = ser.Deserialize<Dictionary<string, object>>(new JsonTextReader(new StringReader(json)));

            Console.WriteLine("Now dictionary!");
            ser.Serialize(new JsonTextWriter(Console.Out), o2);

            Console.WriteLine("Now DataObject!");
            DataObject d2 = new DataObject();
            foreach (string k in o2.Keys)
                d2[k] = o2[k];
            ser.Serialize(new JsonTextWriter(Console.Out), d2);
        }

    }
}
