﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NGinnBPM.Rules.DSL;
using NGinnBPM.Rules;
using System.IO;
using NGinnBPM.Rules.TableDriven;

namespace NGTestApp2
{

    


    public class TestRules
    {
        public static string BaseDir
        {
            get
            {
                return Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "..\\..\\..\\..\\var\\NGinn.Rules\\TestSet1");
            }
        }

        public static void Test()
        {
            RuleEngine re = new RuleEngine();
            re.BaseDirectory = BaseDir;
            re.DetectFileModification = true;
            re.ExecuteRules("rulez.boo", "ala ma kota", Console.Out);
            re.ExecuteRules("rulez.boo", "kota", Console.Out);
            re.ExecuteRules("rulez.boo", "%procenta", Console.Out);
        }

        public static void GenerateRuleset()
        {
            RulesetDef rsd = new RulesetDef();
            rsd.Name = "newrules";
            rsd.Rules = new List<RuleDef>();
            rsd.Rules.Add(new RuleDef
            {
                Id = "RD1",
                Conditions = new string[] { "T.val1.Length > 3", "T.val2 > 0" },
                Actions = new string[] { "print 'kuku'", "R.val1='akuku'" }
            });
            rsd.Rules.Add(new RuleDef
            {
                Id = "RD2",
                Conditions = new string[] { "T.val1.Length < 3", "T.val2  >= 0" },
                Actions = new string[] { "print 'kuku2'", "R.val2 = 199" }
            });
            rsd.Rules.Add(new RuleDef
            {
                Id = "RD3",
                Conditions = new string[] { "T.val1.IndexOf('%') < 0" },
                Actions = new string[] { "print 'kuku3'", "R.val2 = R.val2 + 389" }
            });
            RuleScriptGenerator gen = new RuleScriptGenerator();
            string file = Path.Combine(BaseDir, "newrulez.boo");
            gen.GenerateScript(rsd, file);

            RuleEngine re = new RuleEngine();
            re.BaseDirectory = BaseDir;
            re.DetectFileModification = true;
            
            DynObject dob = new DynObject();
            dob.Set("val1", "ala ma kota");
            dob.Set("val2", 9892);
            re.ExecuteRules("newrulez.boo", dob, dob);
            re.ExecuteRules("newrulez.boo", dob, dob);
            re.ExecuteRules("newrulez.boo", dob, dob);

        }
        

        public static void TestCsvReader(string s)
        {
            NLog.Logger log = NLog.LogManager.GetCurrentClassLogger();
            
            CsvReader cr = new CsvReader(new StringReader(s));
            log.Info("Reading '{0}'", s);
            while (cr.Read())
            {
                log.Info("In line {0} fields {2}: '{1}'", cr.LineNumber, string.Join("','", cr.CurrentRow.ToArray()), cr.CurrentRow.Count);
            }
        }

        public static void TestCsvReader()
        {
            TestCsvReader("");
            TestCsvReader("ala,ma, kota");
            TestCsvReader("ala, ma, \"kota\"");
            TestCsvReader("ala, \"ma\" , \"kota\"");
            TestCsvReader("jola ma, kota  , \"tak\"\"ż\"\"e\" \n");
            TestCsvReader("jola ma, kota  , \"tak\"\"ż\"\"e\" \njola ma , sz\"kota  , \"tak\"\"ż,ź,ę\"\"e2\" ");

            TestCsvReader("ziuta ma, kota  , \"tak\"\"więc\"\"\nsobie z nim żyje\" \njola ma , sz\"kota  , \"tak\"\"ż,ź,ę\"\"e2\" ");

            using(StreamReader sr = new StreamReader("c:\\zeszyt1.csv", Encoding.GetEncoding("windows-1250")))
            {
                CsvReader cr = new CsvReader(sr);
                cr.Separator = ';';
                while (cr.Read())
                {
                    NLog.LogManager.GetLogger("uuu").Info("In line {0} fields {2}: '{1}'", cr.LineNumber, string.Join("','", cr.CurrentRow.ToArray()), cr.CurrentRow.Count);
                }
            };
        }

        public static void TestRuleBuilded()
        {
            RulesetDef rsd;
            using (StreamReader sr = new StreamReader("c:\\zeszyt1.csv", Encoding.GetEncoding("windows-1250")))
            {
                CsvReader cr = new CsvReader(sr);
                cr.Separator = ';';
                TableRulesetBuilder bld = new TableRulesetBuilder();
                rsd = bld.ReadRulesetFromCsv(cr);
            }

            RuleScriptGenerator gen = new RuleScriptGenerator();
            string file = Path.Combine(BaseDir, "zeszyt1.boo");
            gen.GenerateScript(rsd, file);

            RuleEngine re = new RuleEngine();
            re.BaseDirectory = BaseDir;
            re.DetectFileModification = true;
            re.ExecuteRules("zeszyt1.boo", "ala ma kota", Console.Out);
        }

        

        public static void TestRuleEngine1()
        {
            TableDrivenRuleEngine tre = new TableDrivenRuleEngine();
            tre.BaseDirectory = "c:\\";
            DynObject dob = new DynObject();
            dob.Set("From", "A1");
            dob.Set("IsUsable", true);
            dob.Set("IsBusinessCustomer", (Func<bool>) delegate() {
                Console.WriteLine("ISBUSINESSCUSTOMER");
                return false; 
            });
            for (int i = 0; i < 20; i++)
            {
                tre.EvalRules("zeszyt1.csv", dob, dob);
                Console.WriteLine("Proba {0} - pres enter", i);
                Console.ReadLine();
            }
        }


    }


}
